/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: reqtoolsami_init.c,v 1.5 2015/01/18 14:16:53 piru Exp $

    Desc: AmigaOS specific ReqTools initialization code.
    Lang: English.
*/

/****************************************************************************************/

#include <exec/libraries.h>
#include <exec/resident.h>
#include <exec/execbase.h>
#include <exec/tasks.h>
#include <exec/memory.h>
#include <libraries/reqtools.h>
#include <intuition/classes.h>
#include <proto/exec.h>

#include "reqtools_intern.h"
#include "general.h"
#include "rtfuncs.h"

#define VERSION 39
#define REVISION 3

#define NAME_STRING      "reqtools.library"
#ifdef __MORPHOS__
#define VERSION_STRING   "$VER: reqtools 39.3.18 (18.01.2015) MorphOS version by Harry Sintonen\r\n"
#else /* __MORPHOS__ */
#define VERSION_STRING   "$VER: reqtools 39.3 (15.11.2001)\r\n"
#endif /* __MORPHOS__ */

/****************************************************************************************/

extern const char name[];
extern const char version[];
extern const APTR inittable[4];
extern void *const functable[];
extern struct IntReqToolsBase *libinit();
extern const char libend;

/****************************************************************************************/

int entry(void)
{
  return -1;
}

/****************************************************************************************/

const struct Resident ROMTag =
{
  RTC_MATCHWORD,
  (struct Resident *)&ROMTag,
  (APTR)&libend,
#ifdef __MORPHOS__
  RTF_PPC | RTF_EXTENDED |
#endif /* __MORPHOS__ */
  RTF_AUTOINIT,
  VERSION,
  NT_LIBRARY,
  0,
  (char *)name,
  (char *)&version[6],
  (ULONG *)inittable
#ifdef __MORPHOS__
  ,REVISION,
  NULL
#endif /* __MORPHOS__ */
};

/****************************************************************************************/

const char name[]    = NAME_STRING;
const char version[] = VERSION_STRING;

const APTR inittable[4] =
{
    (APTR)sizeof(struct IntReqToolsBase),
    (APTR)functable,
    NULL,
    (APTR)&libinit
};

#ifdef __MORPHOS__
const int __abox__ = 1;
#endif

/****************************************************************************************/

#define extern
#include "globalvars.h"
#undef extern

/****************************************************************************************/

#ifdef __MORPHOS__

struct IntReqToolsBase *libinit(struct IntReqToolsBase *IntReqToolsBase,
				BPTR segList,
				struct ExecBase *sysBase)

#else /* __MORPHOS__ */

struct IntReqToolsBase * SAVEDS ASM libinit(REGPARAM(d0, struct IntReqToolsBase *, IntReqToolsBase),
    	    	    	    	    	    REGPARAM(a0, BPTR, segList))
#endif /* __MORPHOS__ */

{
#ifdef __MORPHOS__
    SysBase = sysBase;
#else /* __MORPHOS__ */
    SysBase = *(struct ExecBase **)4L;
#endif /* __MORPHOS__ */

    /* Workaround for some lame apps (TextView) that call reqtools with
     * bogus A6 - Piru
     */
    ReqToolsBase = (APTR) IntReqToolsBase;

    ReqToolsBase->LibNode.lib_Node.ln_Type  = NT_LIBRARY;
    ReqToolsBase->LibNode.lib_Node.ln_Name  = (char *) name;
    ReqToolsBase->LibNode.lib_Flags         = LIBF_SUMUSED | LIBF_CHANGED;
    ReqToolsBase->LibNode.lib_Version       = VERSION;
    ReqToolsBase->LibNode.lib_Revision      = REVISION;
    ReqToolsBase->LibNode.lib_IdString      = (char *) &version[6];

    return (struct IntReqToolsBase *)RTFuncs_Init(IntReqToolsBase, segList);
}

/****************************************************************************************/

#ifdef USE_STACKSWAP

/****************************************************************************************/

#ifdef _AROS
#error No StackSwap support for AROS yet
#endif

#ifndef __GNUC__
#error Only StackSwap support for GCC compiler for now
#endif

/****************************************************************************************/

struct MyStackSwapStruct
{
    struct StackSwapStruct  sss;
    UBYTE   	    	    *stringbuff;
    LONG    	    	    maxlen;
    char    	    	    *title;
    ULONG   	    	    checksum;
    ULONG   	    	    *value;
    LONG    	    	    mode;
    struct rtReqInfo 	    *reqinfo;
    struct TagItem  	    *taglist;
};

#ifndef MIN_STACK
#define MIN_STACK 4096
#endif

/****************************************************************************************/

static ULONG CheckStack_GetString(UBYTE *stringbuff,
    	    	    	    	  LONG maxlen,
				  char *title,
				  ULONG checksum,
				  ULONG *value,
				  LONG mode,
				  struct rtReqInfo *reqinfo,
				  struct TagItem *taglist,
				  struct IntReqToolsBase *IntReqToolsBase)
{
/* I could not manage to get correct code to be generated when
   using asm("a7") or asm("sp") to access sp!? :-( */

#define sp ((IPTR)(&sss))

    struct MyStackSwapStruct 	sss;
    struct MyStackSwapStruct 	*sssptr asm("a3") = &sss;
    struct Task     	    	*me = FindTask(NULL);
    ULONG   	    	    	retval asm("d5") = 0;

    if ((sp <= (IPTR)me->tc_SPLower) ||
        (sp >= (IPTR)me->tc_SPUpper) ||
	(sp - MIN_STACK < (IPTR)me->tc_SPLower))
    {
    	sssptr->sss.stk_Lower = rtAllocVec(MIN_STACK, MEMF_PUBLIC);
	if (sssptr->sss.stk_Lower)
	{
	    sssptr->sss.stk_Upper = ((IPTR)sssptr->sss.stk_Lower) + MIN_STACK;
	    sssptr->sss.stk_Pointer = (APTR) sssptr->sss.stk_Upper;
	
	    sssptr->stringbuff  = stringbuff;
	    sssptr->maxlen  	= maxlen;
	    sssptr->title   	= title;
	    sssptr->checksum 	= checksum;
	    sssptr->value   	= value;
	    sssptr->mode    	= mode;
	    sssptr->reqinfo 	= reqinfo;
	    sssptr->taglist 	= taglist;
	
	    StackSwap(&sssptr->sss);
	
    	    retval = GetString(sssptr->stringbuff,
	    	    	       sssptr->maxlen,
			       sssptr->title,
			       sssptr->checksum,
			       sssptr->value,
			       sssptr->mode,
			       sssptr->reqinfo,
			       sssptr->taglist,
			       IntReqToolsBase);
			       	
	    StackSwap(&sssptr->sss);
	
	    rtFreeVec(sssptr->sss.stk_Lower);
	}
    }
    else
    {
    	retval = GetString(stringbuff, maxlen, title, checksum, value,
			   mode, reqinfo, taglist, IntReqToolsBase);
    }

    return retval;
}

/****************************************************************************************/

#define GETSTRING CheckStack_GetString

#else

#define GETSTRING GetString

/****************************************************************************************/

#endif /* USE_STACKSWAP */

/****************************************************************************************/

#ifndef __MORPHOS__

ULONG SAVEDS ASM librtEZRequestA(REGPARAM(a1, char *, bodyfmt),
                                 REGPARAM(a2, char *, gadfmt),
			         REGPARAM(a3, struct rtReqInfo *, reqinfo),
			         REGPARAM(a4, APTR, argarray),
			         REGPARAM(a0, struct TagItem *, taglist),
				 REGPARAM(a6, struct IntReqToolsBase *, IntReqToolsBase))
{
    return GETSTRING(bodyfmt,
    		     (LONG)argarray,
		     gadfmt,
		     0,
		     NULL,
		     IS_EZREQUEST,
		     reqinfo,
		     taglist,
		     IntReqToolsBase);
}

/****************************************************************************************/

ULONG SAVEDS ASM librtGetStringA(REGPARAM(a1, UBYTE *, buffer),
    	    	    	    	 REGPARAM(d0, ULONG, maxchars),
				 REGPARAM(a2, char *, title),
				 REGPARAM(a3, struct rtReqInfo *, reqinfo),
				 REGPARAM(a0, struct TagItem *, taglist),
				 REGPARAM(a6, struct IntReqToolsBase *, IntReqToolsBase))
{
    return GETSTRING(buffer,
    		     maxchars,
		     title,
		     0,
		     NULL,
		     ENTER_STRING,
		     reqinfo,
		     taglist,
		     IntReqToolsBase);
}

/****************************************************************************************/

ULONG SAVEDS ASM librtGetLongA(REGPARAM(a1, ULONG *, longptr),
    	    	    	       REGPARAM(a2, char *, title),
			       REGPARAM(a3, struct rtReqInfo *, reqinfo),
			       REGPARAM(a0, struct TagItem *, taglist),
			       REGPARAM(a6, struct IntReqToolsBase *, IntReqToolsBase))
{
    return GETSTRING(NULL,
    		     0,
		     title,
		     0,
		     longptr,
		     ENTER_NUMBER,
		     reqinfo,
		     taglist,
		     IntReqToolsBase);
}

/****************************************************************************************/

/* NOTE: Used by powerpacker.library/ppGetPassword() */

BOOL SAVEDS ASM librtInternalGetPasswordA(REGPARAM(a1, UBYTE *, buffer),
					  REGPARAM(d1, ULONG, checksum),
					  REGPARAM(d2, PWCALLBACKFUNPTR, pwcallback),
					  REGPARAM(a3, struct rtReqInfo *, reqinfo),
					  REGPARAM(a0, struct TagItem *, taglist),
					  REGPARAM(a6, struct IntReqToolsBase *, IntReqToolsBase))
{
    buffer[0] = '\0';

    return GETSTRING(buffer,
    		     16,
    		     "Password",
		     checksum & 0xffff,
		     (ULONG *) pwcallback,
		     CHECK_PASSWORD,
		     reqinfo,
		     taglist,
		     IntReqToolsBase);
}

/****************************************************************************************/

/* NOTE: Used by powerpacker.library/ppEnterPassword */

BOOL SAVEDS ASM librtInternalEnterPasswordA(REGPARAM(a1, UBYTE *, buffer),
					    REGPARAM(d2, PWCALLBACKFUNPTR, pwcallback),
					    REGPARAM(a3, struct rtReqInfo *, reqinfo),
					    REGPARAM(a0, struct TagItem *, taglist),
					    REGPARAM(a6, struct IntReqToolsBase *, IntReqToolsBase))
{
    return GETSTRING(buffer,
    		     16,
    		     "Password",
		     0,
		     (ULONG *) pwcallback,
		     ENTER_PASSWORD,
		     reqinfo,
		     taglist,
		     IntReqToolsBase);
}

#endif /* !__MORPHOS__ */

/****************************************************************************************/

#ifdef __MORPHOS__

extern void LIB_open(void);
extern void LIB_close(void);
extern void LIB_expunge(void);
extern void LIB_null(void);

extern void LIB_rtAllocRequestA(void);
extern void LIB_rtFreeRequest(void);
extern void LIB_rtFreeReqBuffer(void);
extern void LIB_rtChangeReqAttrA(void);
extern void LIB_rtFileRequestA(void);
extern void LIB_rtFreeFileList(void);
extern void LIB_rtEZRequestA(void);
extern void LIB_rtGetStringA(void);
extern void LIB_rtGetLongA(void);
extern void LIB_rtInternalGetPasswordA(void);
extern void LIB_rtInternalEnterPasswordA(void);
extern void LIB_rtFileRequestA(void);
extern void LIB_rtPaletteRequestA(void);
extern void LIB_rtReqHandlerA(void);
extern void LIB_rtSetWaitPointer(void);
extern void LIB_rtGetVScreenSize(void);
extern void LIB_rtSetReqPosition(void);
extern void LIB_rtSpread(void);
extern void LIB_rtScreenToFrontSafely(void);
extern void LIB_rtFileRequestA(void);
extern void LIB_rtCloseWindowSafely(void);
extern void LIB_rtLockWindow(void);
extern void LIB_rtUnlockWindow(void);
extern void LIB_rtLockPrefs(void);
extern void LIB_rtUnlockPrefs(void);

#else /* __MORPHOS__ */

extern void libopen(void);
extern void libclose(void);
extern void libexpunge(void);
extern void libnull(void);

extern void AllocRequestA(void);
extern void FreeRequest(void);
extern void FreeReqBuffer(void);
extern void ChangeReqAttrA(void);
extern void FileRequestA(void);
extern void FreeFileList(void);
extern void PaletteRequestA(void);
extern void GetVScreenSize(void);

#endif /* __MORPHOS__ */


/****************************************************************************************/

void *const functable[] =
{

#ifdef __MORPHOS__

    (void *const) FUNCARRAY_32BIT_NATIVE,

    LIB_open,
    LIB_close,
    LIB_expunge,
    LIB_null,

    LIB_rtAllocRequestA,
    LIB_rtFreeRequest,
    LIB_rtFreeReqBuffer,
    LIB_rtChangeReqAttrA,
    LIB_rtFileRequestA,
    LIB_rtFreeFileList,
    LIB_rtEZRequestA,
    LIB_rtGetStringA,
    LIB_rtGetLongA,
    LIB_rtInternalGetPasswordA,
    LIB_rtInternalEnterPasswordA,
    LIB_rtFileRequestA,
    LIB_rtPaletteRequestA,
    LIB_rtReqHandlerA,
    LIB_rtSetWaitPointer,
    LIB_rtGetVScreenSize,
    LIB_rtSetReqPosition,
    LIB_rtSpread,
    LIB_rtScreenToFrontSafely,
    LIB_rtFileRequestA,
    LIB_rtCloseWindowSafely,
    LIB_rtLockWindow,
    LIB_rtUnlockWindow,
    LIB_rtLockPrefs,
    LIB_rtUnlockPrefs,

#else /* __MORPHOS __ */

    RTFuncs_Open,
    RTFuncs_Close,
    RTFuncs_Expunge,
    RTFuncs_Null,

    AllocRequestA,
    FreeRequest,
    FreeReqBuffer,
    ChangeReqAttrA,
    FileRequestA,
    FreeFileList,
    librtEZRequestA,
    librtGetStringA,
    librtGetLongA,
    librtInternalGetPasswordA,
    librtInternalEnterPasswordA,
    FileRequestA,
    PaletteRequestA,
    RTFuncs_rtReqHandlerA,
    RTFuncs_rtSetWaitPointer,
    GetVScreenSize,
    RTFuncs_rtSetReqPosition,
    RTFuncs_rtSpread,
    RTFuncs_ScreenToFrontSafely,
    FileRequestA,
    RTFuncs_CloseWindowSafely,
    RTFuncs_rtLockWindow,
    RTFuncs_rtUnlockWindow,
    RTFuncs_LockPrefs,
    RTFuncs_UnlockPrefs,

#endif /* __MORPHOS__ */

    (void *const)-1L
};

/****************************************************************************************/

const char libend = 0;

/****************************************************************************************/

